const { logger } = require('./logger')
const { createTimeout } = require('./timeout')
const generateId = require('shortid')

// TODO: Remove this once it's obsolete (when we remove node-jet)
function createQuery (resourceType, constraints) {
  // Setup base query
  let query = {
    path: {
      startsWith: `/${resourceType}/`
    },
    valueField: {
      isAvailable: {
        equals: true
      }
    }
  }

  if (constraints) {
    // Set constraints in query
    Object.keys(constraints).forEach(constraintName => {
      Object.keys(constraints[constraintName]).forEach(predicate => {
        query.valueField[`metadata.${constraintName}.value`] = {
          [predicate]: constraints[constraintName][predicate]
        }
      })
    })
  }

  return query
}

function convertRequest (rawRequest) {
  const requestedResources = rawRequest.requestedResources
  // Check to see if request contains any requested resources
  if (requestedResources) {
    return {
      ...rawRequest,
      requestedResources: Object.keys(requestedResources).reduce(
        (acc, type) => {
          if (type === 'users') {
            requestedResources[type].forEach(requestedResource => {
              acc.push({
                resource: {
                  type
                },
                query: requestedResource.constraints,
                isFulfilled: false
              })
            })
          } else {
            requestedResources[type].forEach(requestedResource => {
              acc.push({
                query: createQuery(type, requestedResource.constraints),
                resource: {
                  type
                },
                isFulfilled: false
              })
            })
          }
          return acc
        },
        []
      )
    }
  } else {
    throw new Error('No requested resources.')
  }
}

function createRequest (peer, rawRequest, timeoutLength = 600000) {
  // Convert raw request
  const convertedRequest = convertRequest(rawRequest)

  const timeout = createTimeout(timeoutLength)

  return {
    ...convertedRequest,
    id: generateId(),
    timeout,
    respond () {
      // Cancel the timeout; it's no longer needed
      timeout.cancel()
      return peer
        .call(this.callbackPath, {
          resources: this.requestedResources.map(
            requestedResource => requestedResource.resource
          )
        })
        .catch(error => {
          logger.error(
            `Error responding to resource request by instructor ${
              this.from
            } - ${error}`
          )
        })
    },
    cancel () {
      try {
        peer.call(this.callbackPath, {
          error: `Error responding to resource request: Timeout of ${timeoutLength}ms has been reached.`
        })
      } catch (error) {
        logger.error(
          `Error calling callbackPath on request cancel (Request from ${
            peer.id
          }) - ${error}`
        )
      }
    }
  }
}

module.exports = { createRequest }
