const { logger, stringify } = require('./helpers/logger')
const jet = require('node-jet')
const sem = require('semaphore')(1)

exports.createJetBroker = function (peer) {
  return {
    startTransaction (callback) {
      sem.take(() => {
        // eslint-disable-next-line
        return callback({
          // CRUD Operations
          get (query) {
            return peer.get(query).catch(error => {
              logger.error(`Failed to get from DB - ${stringify(error)}`)
              logger.debug(`Query: ${stringify(query)}`)
            })
          },
          set (resourceType, { id, value }) {
            return peer.set(`/${resourceType}/${id}`, value).catch(error => {
              logger.error(`Failed to set in DB params - ${error}`)
              logger.debug(
                `Resource: ${stringify({ resourceType, id, value })}`
              )
            })
          },
          add (resourceType, { id, value }) {
            const resource = new jet.State(`/${resourceType}/${id}`, value)
            resource.on('set', newValue => newValue)
            return peer.add(resource)
          },
          remove (resourceType, id) {
            const resource = new jet.State(`/${resourceType}/${id}`)
            return peer.remove(resource)
          }
        }).then(() => sem.leave())
      })
    }
  }
}
